package org.blender.dna;

import java.io.IOException;
import org.cakelab.blender.io.block.Block;
import org.cakelab.blender.io.block.BlockTable;
import org.cakelab.blender.nio.CArrayFacade;
import org.cakelab.blender.nio.CFacade;
import org.cakelab.blender.nio.CMetaData;
import org.cakelab.blender.nio.CPointer;


/**
 * Generated facet for DNA struct type 'ID'.
 * 
 * <h3>Class Documentation</h3>
 * <h4>Java .Blend:</h4>
 * Represents the ID of any element in DNA which 
 * 					 is stored in a library (see {@link Library}). Those 
 * 					 elements are instances of structs which contain an 
 * 					 embedded member of type ID as their first element.<br/>
 * 					 IDs of the same element type are linked in linked list (see {@link Link})
 */

@CMetaData(size32=100, size64=120)
public class ID extends CFacade {

	/**
	 * This is the sdna index of the struct ID.
	 * <p>
	 * It is required when allocating a new block to store data for ID.
	 * </p>
	 * @see {@link org.cakelab.blender.io.dna.internal.StructDNA}
	 * @see {@link org.cakelab.blender.io.block.BlockTable#allocate}
	 */
	public static final int __DNA__SDNA_INDEX = 10;

	/**
	 * Field descriptor (offset) for struct member 'next'.
	 * <h3>Field Documentation</h3>
	 * <h4>Java .Blend:</h4>
	 * reference on the next element in linked list
	 * <h3>Pointer Arithmetics</h3>
	 * <p>
	 * This is how you get a reference on the corresponding field in the struct:
	 * </p>
	 * <pre>
	 * ID id = ...;
	 * CPointer&lt;Object&gt; p = id.__dna__addressof(ID.__DNA__FIELD__next);
	 * CPointer&lt;CPointer&lt;Object&gt;&gt; p_next = p.cast(new Class[]{CPointer.class, Object.class});
	 * </pre>
	 * <h3>Metadata</h3>
	 * <ul>
	 * <li>Field: 'next'</li>
	 * <li>Signature: 'void*'</li>
	 * <li>Actual Size (32bit/64bit): 4/8</li>
	 * </ul>
	 */
	public static final long[] __DNA__FIELD__next = new long[]{0, 0};

	/**
	 * Field descriptor (offset) for struct member 'prev'.
	 * <h3>Field Documentation</h3>
	 * <h4>Java .Blend:</h4>
	 * reference on previous element in linked list
	 * <h3>Pointer Arithmetics</h3>
	 * <p>
	 * This is how you get a reference on the corresponding field in the struct:
	 * </p>
	 * <pre>
	 * ID id = ...;
	 * CPointer&lt;Object&gt; p = id.__dna__addressof(ID.__DNA__FIELD__prev);
	 * CPointer&lt;CPointer&lt;Object&gt;&gt; p_prev = p.cast(new Class[]{CPointer.class, Object.class});
	 * </pre>
	 * <h3>Metadata</h3>
	 * <ul>
	 * <li>Field: 'prev'</li>
	 * <li>Signature: 'void*'</li>
	 * <li>Actual Size (32bit/64bit): 4/8</li>
	 * </ul>
	 */
	public static final long[] __DNA__FIELD__prev = new long[]{4, 8};

	/**
	 * Field descriptor (offset) for struct member 'newid'.
	 * <h3>Pointer Arithmetics</h3>
	 * <p>
	 * This is how you get a reference on the corresponding field in the struct:
	 * </p>
	 * <pre>
	 * ID id = ...;
	 * CPointer&lt;Object&gt; p = id.__dna__addressof(ID.__DNA__FIELD__newid);
	 * CPointer&lt;CPointer&lt;ID&gt;&gt; p_newid = p.cast(new Class[]{CPointer.class, ID.class});
	 * </pre>
	 * <h3>Metadata</h3>
	 * <ul>
	 * <li>Field: 'newid'</li>
	 * <li>Signature: 'ID*'</li>
	 * <li>Actual Size (32bit/64bit): 4/8</li>
	 * </ul>
	 */
	public static final long[] __DNA__FIELD__newid = new long[]{8, 16};

	/**
	 * Field descriptor (offset) for struct member 'lib'.
	 * <h3>Field Documentation</h3>
	 * <h4>Java .Blend:</h4>
	 * Reference on the library this library element belongs to.
	 * <h3>Pointer Arithmetics</h3>
	 * <p>
	 * This is how you get a reference on the corresponding field in the struct:
	 * </p>
	 * <pre>
	 * ID id = ...;
	 * CPointer&lt;Object&gt; p = id.__dna__addressof(ID.__DNA__FIELD__lib);
	 * CPointer&lt;CPointer&lt;Library&gt;&gt; p_lib = p.cast(new Class[]{CPointer.class, Library.class});
	 * </pre>
	 * <h3>Metadata</h3>
	 * <ul>
	 * <li>Field: 'lib'</li>
	 * <li>Signature: 'Library*'</li>
	 * <li>Actual Size (32bit/64bit): 4/8</li>
	 * </ul>
	 */
	public static final long[] __DNA__FIELD__lib = new long[]{12, 24};

	/**
	 * Field descriptor (offset) for struct member 'name'.
	 * <h3>Field Documentation</h3>
	 * <h4>Java .Blend:</h4>
	 * The name of the element. Names follow a certain convention. 
	 * 				    <p>
	 * 				    Names of library elements which have an specific block code, such as 
	 * 				    Mesh (ME) and Tex (ME), always have the first two ASCII characters of
	 * 				    the block code as its prefix. Data of blocks with unspecified block 
	 * 				    code (i.e. {@link BlockCodes#ID_DATA}) have other names.
	 * 				    </p>
	 * 				    <b>A few examples of names:</b>
	 * 					<table border="1">
	 * 					<tr><th>Element type</th><th>Name prefix</th><tr>
	 * 					<tr><td>{@link BlenderObject}</td><td>"OB"</td></tr>
	 * 					<tr><td>{@link Mesh}</td><td>"ME"</td></tr>
	 * 					<tr><td>{@link Material}</td><td>"MA"</td></tr>
	 * 					<tr><td>{@link Tex}</td><td>"TE"</td></tr>
	 * 					<tr><td>{@link CustomDataLayer} for {@link MPoly}</td><td>"NGon Face"</td></tr>
	 * 					<tr><td>{@link CustomDataLayer} for {@link MLoop}</td><td>"NGon Face-Vertex"</td></tr>
	 * 					</table><h4>Blender Python API:</h4>
	 * Unique datablock ID name
	 * <h3>Pointer Arithmetics</h3>
	 * <p>
	 * This is how you get a reference on the corresponding field in the struct:
	 * </p>
	 * <pre>
	 * ID id = ...;
	 * CPointer&lt;Object&gt; p = id.__dna__addressof(ID.__DNA__FIELD__name);
	 * CPointer&lt;CArrayFacade&lt;Byte&gt;&gt; p_name = p.cast(new Class[]{CArrayFacade.class, Byte.class});
	 * </pre>
	 * <h3>Metadata</h3>
	 * <ul>
	 * <li>Field: 'name'</li>
	 * <li>Signature: 'char[66]'</li>
	 * <li>Actual Size (32bit/64bit): 66/66</li>
	 * </ul>
	 */
	public static final long[] __DNA__FIELD__name = new long[]{16, 32};

	/**
	 * Field descriptor (offset) for struct member 'pad'.
	 * <h3>Field Documentation</h3>
	 * <h4>Java .Blend:</h4>
	 * padding for alignment
	 * <h3>Pointer Arithmetics</h3>
	 * <p>
	 * This is how you get a reference on the corresponding field in the struct:
	 * </p>
	 * <pre>
	 * ID id = ...;
	 * CPointer&lt;Object&gt; p = id.__dna__addressof(ID.__DNA__FIELD__pad);
	 * CPointer&lt;Short&gt; p_pad = p.cast(new Class[]{Short.class});
	 * </pre>
	 * <h3>Metadata</h3>
	 * <ul>
	 * <li>Field: 'pad'</li>
	 * <li>Signature: 'short'</li>
	 * <li>Actual Size (32bit/64bit): 2/2</li>
	 * </ul>
	 */
	public static final long[] __DNA__FIELD__pad = new long[]{82, 98};

	/**
	 * Field descriptor (offset) for struct member 'us'.
	 * <h3>Field Documentation</h3>
	 * <h4>Java .Blend:</h4>
	 * Reference counter, counts number of references on the corresponding element. If 0, the element will not be stored in a file.
	 * <h3>Pointer Arithmetics</h3>
	 * <p>
	 * This is how you get a reference on the corresponding field in the struct:
	 * </p>
	 * <pre>
	 * ID id = ...;
	 * CPointer&lt;Object&gt; p = id.__dna__addressof(ID.__DNA__FIELD__us);
	 * CPointer&lt;Short&gt; p_us = p.cast(new Class[]{Short.class});
	 * </pre>
	 * <h3>Metadata</h3>
	 * <ul>
	 * <li>Field: 'us'</li>
	 * <li>Signature: 'short'</li>
	 * <li>Actual Size (32bit/64bit): 2/2</li>
	 * </ul>
	 */
	public static final long[] __DNA__FIELD__us = new long[]{84, 100};

	/**
	 * Field descriptor (offset) for struct member 'flag'.
	 * <h3>Field Documentation</h3>
	 * <h4>Java .Blend:</h4>
	 * In files usually 1024. One of: <table border="1">
	 * 					<tr><td>LIB_LOCAL</td><td>0</td><td></td><tr>
	 * 					<tr><td>LIB_EXTERN</td><td>1</td><td></td><tr>
	 * 					<tr><td>LIB_INDIRECT</td><td>2</td><td></td><tr>
	 * 					<tr><td>LIB_NEED_EXPAND</td><td>8</td><td></td><tr>
	 * 					<tr><td>LIB_TESTEXT</td><td>(LIB_NEED_EXPAND | LIB_EXTERN)</td><td></td><tr>
	 * 					<tr><td>LIB_TESTIND</td><td>(LIB_NEED_EXPAND | LIB_INDIRECT)</td><td></td><tr>
	 * 					<tr><td>LIB_READ</td><td>16</td><td></td><tr>
	 * 					<tr><td>LIB_NEED_LINK</td><td>32</td><td></td><tr>
	 * 					<tr><td>LIB_NEW</td><td>256</td><td></td><tr>
	 * 					<tr><td>LIB_FAKEUSER</td><td>512</td><td></td><tr>
	 * 					<tr><td>LIB_DOIT</td><td>1024</td><td>free test flag</td><tr>
	 * 					<tr><td>LIB_PRE_EXISTING</td><td>2048</td><td>tag existing data before linking so we know what is new</td><tr>
	 * 					<tr><td>LIB_ID_RECALC</td><td>4096</td><td>runtime</td><tr>
	 * 					<tr><td>LIB_ID_RECALC_DATA</td><td>8192</td><td>runtime</td><tr>
	 * 					</table>
	 * 				<h4>Blender Source Code:</h4>
	 * LIB_... flags report on status of the datablock this ID belongs to.
	 * <h3>Pointer Arithmetics</h3>
	 * <p>
	 * This is how you get a reference on the corresponding field in the struct:
	 * </p>
	 * <pre>
	 * ID id = ...;
	 * CPointer&lt;Object&gt; p = id.__dna__addressof(ID.__DNA__FIELD__flag);
	 * CPointer&lt;Short&gt; p_flag = p.cast(new Class[]{Short.class});
	 * </pre>
	 * <h3>Metadata</h3>
	 * <ul>
	 * <li>Field: 'flag'</li>
	 * <li>Signature: 'short'</li>
	 * <li>Actual Size (32bit/64bit): 2/2</li>
	 * </ul>
	 */
	public static final long[] __DNA__FIELD__flag = new long[]{86, 102};

	/**
	 * Field descriptor (offset) for struct member 'icon_id'.
	 * <h3>Field Documentation</h3>
	 * <h4>Java .Blend:</h4>
	 * Always 0 in files. Will be recalculated base on the library element type when loaded.
	 * <h3>Pointer Arithmetics</h3>
	 * <p>
	 * This is how you get a reference on the corresponding field in the struct:
	 * </p>
	 * <pre>
	 * ID id = ...;
	 * CPointer&lt;Object&gt; p = id.__dna__addressof(ID.__DNA__FIELD__icon_id);
	 * CPointer&lt;Integer&gt; p_icon_id = p.cast(new Class[]{Integer.class});
	 * </pre>
	 * <h3>Metadata</h3>
	 * <ul>
	 * <li>Field: 'icon_id'</li>
	 * <li>Signature: 'int'</li>
	 * <li>Actual Size (32bit/64bit): 4/4</li>
	 * </ul>
	 */
	public static final long[] __DNA__FIELD__icon_id = new long[]{88, 104};

	/**
	 * Field descriptor (offset) for struct member 'pad2'.
	 * <h3>Field Documentation</h3>
	 * <h4>Java .Blend:</h4>
	 * padding for alignment
	 * <h3>Pointer Arithmetics</h3>
	 * <p>
	 * This is how you get a reference on the corresponding field in the struct:
	 * </p>
	 * <pre>
	 * ID id = ...;
	 * CPointer&lt;Object&gt; p = id.__dna__addressof(ID.__DNA__FIELD__pad2);
	 * CPointer&lt;Integer&gt; p_pad2 = p.cast(new Class[]{Integer.class});
	 * </pre>
	 * <h3>Metadata</h3>
	 * <ul>
	 * <li>Field: 'pad2'</li>
	 * <li>Signature: 'int'</li>
	 * <li>Actual Size (32bit/64bit): 4/4</li>
	 * </ul>
	 */
	public static final long[] __DNA__FIELD__pad2 = new long[]{92, 108};

	/**
	 * Field descriptor (offset) for struct member 'properties'.
	 * <h3>Field Documentation</h3>
	 * <h4>Java .Blend:</h4>
	 * List of associated properties.
	 * <h3>Pointer Arithmetics</h3>
	 * <p>
	 * This is how you get a reference on the corresponding field in the struct:
	 * </p>
	 * <pre>
	 * ID id = ...;
	 * CPointer&lt;Object&gt; p = id.__dna__addressof(ID.__DNA__FIELD__properties);
	 * CPointer&lt;CPointer&lt;IDProperty&gt;&gt; p_properties = p.cast(new Class[]{CPointer.class, IDProperty.class});
	 * </pre>
	 * <h3>Metadata</h3>
	 * <ul>
	 * <li>Field: 'properties'</li>
	 * <li>Signature: 'IDProperty*'</li>
	 * <li>Actual Size (32bit/64bit): 4/8</li>
	 * </ul>
	 */
	public static final long[] __DNA__FIELD__properties = new long[]{96, 112};

	public ID(long __address, Block __block, BlockTable __blockTable) {
		super(__address, __block, __blockTable);
	}

	protected ID(ID that) {
		super(that.__io__address, that.__io__block, that.__io__blockTable);
	}

	/**
	 * Get method for struct member 'next'.
	 * <h3>Field Documentation</h3>
	 * <h4>Java .Blend:</h4>
	 * reference on the next element in linked list
	 * @see #__DNA__FIELD__next
	 */
	
	public CPointer<Object> getNext() throws IOException
	{
		long __dna__targetAddress;
		if ((__io__pointersize == 8)) {
			__dna__targetAddress = __io__block.readLong(__io__address + 0);
		} else {
			__dna__targetAddress = __io__block.readLong(__io__address + 0);
		}
		Class<?>[] __dna__targetTypes = new Class[]{Object.class};
		return new CPointer<Object>(__dna__targetAddress, __dna__targetTypes, __io__blockTable.getBlock(__dna__targetAddress, -1), __io__blockTable);
	}

	/**
	 * Set method for struct member 'next'.
	 * <h3>Field Documentation</h3>
	 * <h4>Java .Blend:</h4>
	 * reference on the next element in linked list
	 * @see #__DNA__FIELD__next
	 */
	
	public void setNext(CPointer<Object> next) throws IOException
	{
		long __address = ((next == null) ? 0 : next.getAddress());
		if ((__io__pointersize == 8)) {
			__io__block.writeLong(__io__address + 0, __address);
		} else {
			__io__block.writeLong(__io__address + 0, __address);
		}
	}

	/**
	 * Get method for struct member 'prev'.
	 * <h3>Field Documentation</h3>
	 * <h4>Java .Blend:</h4>
	 * reference on previous element in linked list
	 * @see #__DNA__FIELD__prev
	 */
	
	public CPointer<Object> getPrev() throws IOException
	{
		long __dna__targetAddress;
		if ((__io__pointersize == 8)) {
			__dna__targetAddress = __io__block.readLong(__io__address + 8);
		} else {
			__dna__targetAddress = __io__block.readLong(__io__address + 4);
		}
		Class<?>[] __dna__targetTypes = new Class[]{Object.class};
		return new CPointer<Object>(__dna__targetAddress, __dna__targetTypes, __io__blockTable.getBlock(__dna__targetAddress, -1), __io__blockTable);
	}

	/**
	 * Set method for struct member 'prev'.
	 * <h3>Field Documentation</h3>
	 * <h4>Java .Blend:</h4>
	 * reference on previous element in linked list
	 * @see #__DNA__FIELD__prev
	 */
	
	public void setPrev(CPointer<Object> prev) throws IOException
	{
		long __address = ((prev == null) ? 0 : prev.getAddress());
		if ((__io__pointersize == 8)) {
			__io__block.writeLong(__io__address + 8, __address);
		} else {
			__io__block.writeLong(__io__address + 4, __address);
		}
	}

	/**
	 * Get method for struct member 'newid'.
	 * @see #__DNA__FIELD__newid
	 */
	
	public CPointer<ID> getNewid() throws IOException
	{
		long __dna__targetAddress;
		if ((__io__pointersize == 8)) {
			__dna__targetAddress = __io__block.readLong(__io__address + 16);
		} else {
			__dna__targetAddress = __io__block.readLong(__io__address + 8);
		}
		Class<?>[] __dna__targetTypes = new Class[]{ID.class};
		return new CPointer<ID>(__dna__targetAddress, __dna__targetTypes, __io__blockTable.getBlock(__dna__targetAddress, ID.__DNA__SDNA_INDEX), __io__blockTable);
	}

	/**
	 * Set method for struct member 'newid'.
	 * @see #__DNA__FIELD__newid
	 */
	
	public void setNewid(CPointer<ID> newid) throws IOException
	{
		long __address = ((newid == null) ? 0 : newid.getAddress());
		if ((__io__pointersize == 8)) {
			__io__block.writeLong(__io__address + 16, __address);
		} else {
			__io__block.writeLong(__io__address + 8, __address);
		}
	}

	/**
	 * Get method for struct member 'lib'.
	 * <h3>Field Documentation</h3>
	 * <h4>Java .Blend:</h4>
	 * Reference on the library this library element belongs to.
	 * @see #__DNA__FIELD__lib
	 */
	
	public CPointer<Library> getLib() throws IOException
	{
		long __dna__targetAddress;
		if ((__io__pointersize == 8)) {
			__dna__targetAddress = __io__block.readLong(__io__address + 24);
		} else {
			__dna__targetAddress = __io__block.readLong(__io__address + 12);
		}
		Class<?>[] __dna__targetTypes = new Class[]{Library.class};
		return new CPointer<Library>(__dna__targetAddress, __dna__targetTypes, __io__blockTable.getBlock(__dna__targetAddress, Library.__DNA__SDNA_INDEX), __io__blockTable);
	}

	/**
	 * Set method for struct member 'lib'.
	 * <h3>Field Documentation</h3>
	 * <h4>Java .Blend:</h4>
	 * Reference on the library this library element belongs to.
	 * @see #__DNA__FIELD__lib
	 */
	
	public void setLib(CPointer<Library> lib) throws IOException
	{
		long __address = ((lib == null) ? 0 : lib.getAddress());
		if ((__io__pointersize == 8)) {
			__io__block.writeLong(__io__address + 24, __address);
		} else {
			__io__block.writeLong(__io__address + 12, __address);
		}
	}

	/**
	 * Get method for struct member 'name'.
	 * <h3>Field Documentation</h3>
	 * <h4>Java .Blend:</h4>
	 * The name of the element. Names follow a certain convention. 
	 * 				    <p>
	 * 				    Names of library elements which have an specific block code, such as 
	 * 				    Mesh (ME) and Tex (ME), always have the first two ASCII characters of
	 * 				    the block code as its prefix. Data of blocks with unspecified block 
	 * 				    code (i.e. {@link BlockCodes#ID_DATA}) have other names.
	 * 				    </p>
	 * 				    <b>A few examples of names:</b>
	 * 					<table border="1">
	 * 					<tr><th>Element type</th><th>Name prefix</th><tr>
	 * 					<tr><td>{@link BlenderObject}</td><td>"OB"</td></tr>
	 * 					<tr><td>{@link Mesh}</td><td>"ME"</td></tr>
	 * 					<tr><td>{@link Material}</td><td>"MA"</td></tr>
	 * 					<tr><td>{@link Tex}</td><td>"TE"</td></tr>
	 * 					<tr><td>{@link CustomDataLayer} for {@link MPoly}</td><td>"NGon Face"</td></tr>
	 * 					<tr><td>{@link CustomDataLayer} for {@link MLoop}</td><td>"NGon Face-Vertex"</td></tr>
	 * 					</table><h4>Blender Python API:</h4>
	 * Unique datablock ID name
	 * @see #__DNA__FIELD__name
	 */
	
	public CArrayFacade<Byte> getName() throws IOException
	{
		Class<?>[] __dna__targetTypes = new Class[]{Byte.class};
		int[] __dna__dimensions = new int[]{
			66
		};
		if ((__io__pointersize == 8)) {
			return new CArrayFacade<Byte>(__io__address + 32, __dna__targetTypes, __dna__dimensions, __io__block, __io__blockTable);
		} else {
			return new CArrayFacade<Byte>(__io__address + 16, __dna__targetTypes, __dna__dimensions, __io__block, __io__blockTable);
		}
	}

	/**
	 * Set method for struct member 'name'.
	 * <h3>Field Documentation</h3>
	 * <h4>Java .Blend:</h4>
	 * The name of the element. Names follow a certain convention. 
	 * 				    <p>
	 * 				    Names of library elements which have an specific block code, such as 
	 * 				    Mesh (ME) and Tex (ME), always have the first two ASCII characters of
	 * 				    the block code as its prefix. Data of blocks with unspecified block 
	 * 				    code (i.e. {@link BlockCodes#ID_DATA}) have other names.
	 * 				    </p>
	 * 				    <b>A few examples of names:</b>
	 * 					<table border="1">
	 * 					<tr><th>Element type</th><th>Name prefix</th><tr>
	 * 					<tr><td>{@link BlenderObject}</td><td>"OB"</td></tr>
	 * 					<tr><td>{@link Mesh}</td><td>"ME"</td></tr>
	 * 					<tr><td>{@link Material}</td><td>"MA"</td></tr>
	 * 					<tr><td>{@link Tex}</td><td>"TE"</td></tr>
	 * 					<tr><td>{@link CustomDataLayer} for {@link MPoly}</td><td>"NGon Face"</td></tr>
	 * 					<tr><td>{@link CustomDataLayer} for {@link MLoop}</td><td>"NGon Face-Vertex"</td></tr>
	 * 					</table><h4>Blender Python API:</h4>
	 * Unique datablock ID name
	 * @see #__DNA__FIELD__name
	 */
	
	public void setName(CArrayFacade<Byte> name) throws IOException
	{
		long __dna__offset;
		if ((__io__pointersize == 8)) {
			__dna__offset = 32;
		} else {
			__dna__offset = 16;
		}
		if (__io__equals(name, __io__address + __dna__offset)) {
			return;
		} else if (__io__same__encoding(this, name)) {
			__io__native__copy(__io__block, __io__address + __dna__offset, name);
		} else {
			__io__generic__copy( getName(), name);
		}
	}

	/**
	 * Get method for struct member 'pad'.
	 * <h3>Field Documentation</h3>
	 * <h4>Java .Blend:</h4>
	 * padding for alignment
	 * @see #__DNA__FIELD__pad
	 */
	
	public short getPad() throws IOException
	{
		if ((__io__pointersize == 8)) {
			return __io__block.readShort(__io__address + 98);
		} else {
			return __io__block.readShort(__io__address + 82);
		}
	}

	/**
	 * Set method for struct member 'pad'.
	 * <h3>Field Documentation</h3>
	 * <h4>Java .Blend:</h4>
	 * padding for alignment
	 * @see #__DNA__FIELD__pad
	 */
	
	public void setPad(short pad) throws IOException
	{
		if ((__io__pointersize == 8)) {
			__io__block.writeShort(__io__address + 98, pad);
		} else {
			__io__block.writeShort(__io__address + 82, pad);
		}
	}

	/**
	 * Get method for struct member 'us'.
	 * <h3>Field Documentation</h3>
	 * <h4>Java .Blend:</h4>
	 * Reference counter, counts number of references on the corresponding element. If 0, the element will not be stored in a file.
	 * @see #__DNA__FIELD__us
	 */
	
	public short getUs() throws IOException
	{
		if ((__io__pointersize == 8)) {
			return __io__block.readShort(__io__address + 100);
		} else {
			return __io__block.readShort(__io__address + 84);
		}
	}

	/**
	 * Set method for struct member 'us'.
	 * <h3>Field Documentation</h3>
	 * <h4>Java .Blend:</h4>
	 * Reference counter, counts number of references on the corresponding element. If 0, the element will not be stored in a file.
	 * @see #__DNA__FIELD__us
	 */
	
	public void setUs(short us) throws IOException
	{
		if ((__io__pointersize == 8)) {
			__io__block.writeShort(__io__address + 100, us);
		} else {
			__io__block.writeShort(__io__address + 84, us);
		}
	}

	/**
	 * Get method for struct member 'flag'.
	 * <h3>Field Documentation</h3>
	 * <h4>Java .Blend:</h4>
	 * In files usually 1024. One of: <table border="1">
	 * 					<tr><td>LIB_LOCAL</td><td>0</td><td></td><tr>
	 * 					<tr><td>LIB_EXTERN</td><td>1</td><td></td><tr>
	 * 					<tr><td>LIB_INDIRECT</td><td>2</td><td></td><tr>
	 * 					<tr><td>LIB_NEED_EXPAND</td><td>8</td><td></td><tr>
	 * 					<tr><td>LIB_TESTEXT</td><td>(LIB_NEED_EXPAND | LIB_EXTERN)</td><td></td><tr>
	 * 					<tr><td>LIB_TESTIND</td><td>(LIB_NEED_EXPAND | LIB_INDIRECT)</td><td></td><tr>
	 * 					<tr><td>LIB_READ</td><td>16</td><td></td><tr>
	 * 					<tr><td>LIB_NEED_LINK</td><td>32</td><td></td><tr>
	 * 					<tr><td>LIB_NEW</td><td>256</td><td></td><tr>
	 * 					<tr><td>LIB_FAKEUSER</td><td>512</td><td></td><tr>
	 * 					<tr><td>LIB_DOIT</td><td>1024</td><td>free test flag</td><tr>
	 * 					<tr><td>LIB_PRE_EXISTING</td><td>2048</td><td>tag existing data before linking so we know what is new</td><tr>
	 * 					<tr><td>LIB_ID_RECALC</td><td>4096</td><td>runtime</td><tr>
	 * 					<tr><td>LIB_ID_RECALC_DATA</td><td>8192</td><td>runtime</td><tr>
	 * 					</table>
	 * 				<h4>Blender Source Code:</h4>
	 * LIB_... flags report on status of the datablock this ID belongs to.
	 * @see #__DNA__FIELD__flag
	 */
	
	public short getFlag() throws IOException
	{
		if ((__io__pointersize == 8)) {
			return __io__block.readShort(__io__address + 102);
		} else {
			return __io__block.readShort(__io__address + 86);
		}
	}

	/**
	 * Set method for struct member 'flag'.
	 * <h3>Field Documentation</h3>
	 * <h4>Java .Blend:</h4>
	 * In files usually 1024. One of: <table border="1">
	 * 					<tr><td>LIB_LOCAL</td><td>0</td><td></td><tr>
	 * 					<tr><td>LIB_EXTERN</td><td>1</td><td></td><tr>
	 * 					<tr><td>LIB_INDIRECT</td><td>2</td><td></td><tr>
	 * 					<tr><td>LIB_NEED_EXPAND</td><td>8</td><td></td><tr>
	 * 					<tr><td>LIB_TESTEXT</td><td>(LIB_NEED_EXPAND | LIB_EXTERN)</td><td></td><tr>
	 * 					<tr><td>LIB_TESTIND</td><td>(LIB_NEED_EXPAND | LIB_INDIRECT)</td><td></td><tr>
	 * 					<tr><td>LIB_READ</td><td>16</td><td></td><tr>
	 * 					<tr><td>LIB_NEED_LINK</td><td>32</td><td></td><tr>
	 * 					<tr><td>LIB_NEW</td><td>256</td><td></td><tr>
	 * 					<tr><td>LIB_FAKEUSER</td><td>512</td><td></td><tr>
	 * 					<tr><td>LIB_DOIT</td><td>1024</td><td>free test flag</td><tr>
	 * 					<tr><td>LIB_PRE_EXISTING</td><td>2048</td><td>tag existing data before linking so we know what is new</td><tr>
	 * 					<tr><td>LIB_ID_RECALC</td><td>4096</td><td>runtime</td><tr>
	 * 					<tr><td>LIB_ID_RECALC_DATA</td><td>8192</td><td>runtime</td><tr>
	 * 					</table>
	 * 				<h4>Blender Source Code:</h4>
	 * LIB_... flags report on status of the datablock this ID belongs to.
	 * @see #__DNA__FIELD__flag
	 */
	
	public void setFlag(short flag) throws IOException
	{
		if ((__io__pointersize == 8)) {
			__io__block.writeShort(__io__address + 102, flag);
		} else {
			__io__block.writeShort(__io__address + 86, flag);
		}
	}

	/**
	 * Get method for struct member 'icon_id'.
	 * <h3>Field Documentation</h3>
	 * <h4>Java .Blend:</h4>
	 * Always 0 in files. Will be recalculated base on the library element type when loaded.
	 * @see #__DNA__FIELD__icon_id
	 */
	
	public int getIcon_id() throws IOException
	{
		if ((__io__pointersize == 8)) {
			return __io__block.readInt(__io__address + 104);
		} else {
			return __io__block.readInt(__io__address + 88);
		}
	}

	/**
	 * Set method for struct member 'icon_id'.
	 * <h3>Field Documentation</h3>
	 * <h4>Java .Blend:</h4>
	 * Always 0 in files. Will be recalculated base on the library element type when loaded.
	 * @see #__DNA__FIELD__icon_id
	 */
	
	public void setIcon_id(int icon_id) throws IOException
	{
		if ((__io__pointersize == 8)) {
			__io__block.writeInt(__io__address + 104, icon_id);
		} else {
			__io__block.writeInt(__io__address + 88, icon_id);
		}
	}

	/**
	 * Get method for struct member 'pad2'.
	 * <h3>Field Documentation</h3>
	 * <h4>Java .Blend:</h4>
	 * padding for alignment
	 * @see #__DNA__FIELD__pad2
	 */
	
	public int getPad2() throws IOException
	{
		if ((__io__pointersize == 8)) {
			return __io__block.readInt(__io__address + 108);
		} else {
			return __io__block.readInt(__io__address + 92);
		}
	}

	/**
	 * Set method for struct member 'pad2'.
	 * <h3>Field Documentation</h3>
	 * <h4>Java .Blend:</h4>
	 * padding for alignment
	 * @see #__DNA__FIELD__pad2
	 */
	
	public void setPad2(int pad2) throws IOException
	{
		if ((__io__pointersize == 8)) {
			__io__block.writeInt(__io__address + 108, pad2);
		} else {
			__io__block.writeInt(__io__address + 92, pad2);
		}
	}

	/**
	 * Get method for struct member 'properties'.
	 * <h3>Field Documentation</h3>
	 * <h4>Java .Blend:</h4>
	 * List of associated properties.
	 * @see #__DNA__FIELD__properties
	 */
	
	public CPointer<IDProperty> getProperties() throws IOException
	{
		long __dna__targetAddress;
		if ((__io__pointersize == 8)) {
			__dna__targetAddress = __io__block.readLong(__io__address + 112);
		} else {
			__dna__targetAddress = __io__block.readLong(__io__address + 96);
		}
		Class<?>[] __dna__targetTypes = new Class[]{IDProperty.class};
		return new CPointer<IDProperty>(__dna__targetAddress, __dna__targetTypes, __io__blockTable.getBlock(__dna__targetAddress, IDProperty.__DNA__SDNA_INDEX), __io__blockTable);
	}

	/**
	 * Set method for struct member 'properties'.
	 * <h3>Field Documentation</h3>
	 * <h4>Java .Blend:</h4>
	 * List of associated properties.
	 * @see #__DNA__FIELD__properties
	 */
	
	public void setProperties(CPointer<IDProperty> properties) throws IOException
	{
		long __address = ((properties == null) ? 0 : properties.getAddress());
		if ((__io__pointersize == 8)) {
			__io__block.writeLong(__io__address + 112, __address);
		} else {
			__io__block.writeLong(__io__address + 96, __address);
		}
	}

	/**
	 * Instantiates a pointer on this instance.
	 */
	public CPointer<ID> __io__addressof() {
		return new CPointer<ID>(__io__address, new Class[]{ID.class}, __io__block, __io__blockTable);
	}

}
